/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.car.impl;

import com.yiring.app.domain.car.Car;
import com.yiring.app.domain.car.CarRepository;
import com.yiring.app.param.car.CarParam;
import com.yiring.app.param.car.CarQueryParam;
import com.yiring.app.service.car.CarService;
import com.yiring.app.vo.car.CarVo;
import com.yiring.auth.domain.role.Role;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.util.StrUtils;
import com.yiring.common.util.date.DateUtils;
import com.yiring.common.vo.PageVo;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

/**
 * @author tzl
 * 2022/4/11 13:58
 */
@Service
public class CarServiceImpl implements CarService {

    @Resource
    CarRepository carRepository;

    @Override
    public Result<Long> saveCarInfo(CarParam carParam) {
        // 检查车牌号是否已存在
        if (hasCarInfoByCarNum(carParam.getCarNum())) {
            return Result.no(Status.BAD_REQUEST, "您的车牌号已预约登记");
        }
        // 检查司机手机号码号是否已存在
        if (hasCarInfoByMobile(carParam.getDriverMobile())) {
            return Result.no(Status.BAD_REQUEST, "您的联系方式已预约登记");
        }
        Car car = Car
            .builder()
            .labelCardStatus("1")
            .carNum(carParam.getCarNum())
            .carType(carParam.getCarType())
            .driverMobile(carParam.getDriverMobile())
            .driverName(carParam.getDriverName())
            .intervieweeId(carParam.getIntervieweeId())
            .orgId(carParam.getOrgId())
            .reason(carParam.getReason())
            .createTime(LocalDateTime.now())
            .build();
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<Long> updateCarInfo(CarParam carParam, IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Car car = optional.get();
        //如果标签卡在使用中无法修改
        if (StringUtils.isNotBlank(car.getLabelCardStatus()) && "2".equals(car.getLabelCardStatus())) {
            return Result.no(Status.BAD_REQUEST, "标签卡使用中,信息无法修改");
        }
        if (!car.getCarNum().equals(carParam.getCarNum())) {
            // 当修改车牌号时才检查重复
            if (hasCarInfoByCarNum(carParam.getCarNum())) {
                return Result.no(Status.BAD_REQUEST, "您的车牌号已预约登记");
            }
        }
        if (!car.getDriverMobile().equals(carParam.getDriverMobile())) {
            // 当修改联系方式时才检查重复
            if (hasCarInfoByMobile(carParam.getDriverMobile())) {
                return Result.no(Status.BAD_REQUEST, "您的联系方式已预约登记");
            }
        }
        BeanUtils.copyProperties(carParam, car);
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<Long> cardRec(IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Car car = optional.get();
        car.setLabelCardStatus("3");
        car.setCardRecTime(LocalDateTime.now());
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<Long> cardSend(IdParam idParam, String labelCard) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Car car = optional.get();
        car.setLabelCardStatus("2");
        car.setCardSendTime(LocalDateTime.now());
        car.setLabelCard(labelCard);
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<String> deleteCarInfo(IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        //正在使用中的信息无法删除
        Car entity = optional.get();
        if ("2".equals(entity.getLabelCardStatus())) {
            return Result.no(Status.BAD_REQUEST, "标签卡使用中,信息无法删除");
        }
        carRepository.delete(entity);
        return Result.ok();
    }

    @Override
    public Result<CarVo> getCarInfo(IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Car entity = optional.get();
        CarVo vo = new CarVo();
        BeanUtils.copyProperties(entity, vo, Role.Fields.permissions);
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<CarVo>> pageCarInfo(CarQueryParam carParam) {
        PageVo<CarVo> resultVo = null;
        //排序
        Sort sort = Sort.by(Sort.Order.desc(Car.Fields.createTime));
        //如果传分页参数则分页,否查全部数据
        if (Objects.nonNull(carParam.getPageNo()) && Objects.nonNull(carParam.getPageSize())) {
            //分页
            Pageable pageable = PageRequest.of(carParam.getPageNo() - 1, carParam.getPageSize());
            Page<Car> all = carRepository.findAll(condition(carParam), pageable);
            List<CarVo> data = all
                .get()
                .map(car -> {
                    CarVo vo = new CarVo();
                    BeanUtils.copyProperties(car, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, all.getTotalElements());
        } else {
            List<Car> all = carRepository.findAll(condition(carParam), sort);
            List<CarVo> data = all
                .stream()
                .map(car -> {
                    CarVo vo = new CarVo();
                    BeanUtils.copyProperties(car, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, data.size());
        }
        return Result.ok(resultVo);
    }

    @Override
    public List<Car> export(CarQueryParam carParam) {
        return carRepository.findAll(condition(carParam));
    }

    @Override
    public boolean hasCarInfoByCarNum(String param) {
        Car entity = Car.builder().carNum(param).labelCardStatus("1").build();
        long count = carRepository.count(Example.of(entity));
        entity = Car.builder().carNum(param).labelCardStatus("2").build();
        long count1 = carRepository.count(Example.of(entity));
        return count + count1 > 0;
    }

    @Override
    public boolean hasCarInfoByMobile(String param) {
        Car entity = Car.builder().driverMobile(param).labelCardStatus("1").build();
        long count = carRepository.count(Example.of(entity));
        entity = Car.builder().driverMobile(param).labelCardStatus("2").build();
        long count1 = carRepository.count(Example.of(entity));
        return count + count1 > 0;
    }

    @Override
    public Specification<Car> condition(CarQueryParam carParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            //查询条件
            if (StrUtils.isNotBlank(carParam.getCarNum())) {
                //车牌号码查询
                list.add(criteriaBuilder.like(root.get("carNum").as(String.class), "%" + carParam.getCarNum() + "%"));
            }

            if (StrUtils.isNotBlank(carParam.getCarType())) {
                //车辆类型查询
                list.add(criteriaBuilder.like(root.get("carType").as(String.class), "%" + carParam.getCarType() + "%"));
            }

            if (StrUtils.isNotBlank(carParam.getDriverName())) {
                //司机名称查询
                list.add(
                    criteriaBuilder.like(root.get("driverName").as(String.class), "%" + carParam.getDriverName() + "%")
                );
            }

            if (StrUtils.isNotBlank(carParam.getDriverMobile())) {
                //司机电话号码查询
                list.add(
                    criteriaBuilder.like(
                        root.get("driverMobile").as(String.class),
                        "%" + carParam.getDriverMobile() + "%"
                    )
                );
            }

            if (StrUtils.isNotBlank(carParam.getOrgId())) {
                //所属单位查询
                list.add(criteriaBuilder.like(root.get("orgId").as(String.class), "%" + carParam.getOrgId() + "%"));
            }

            if (StrUtils.isNotBlank(carParam.getReason())) {
                //来访原因查询
                list.add(criteriaBuilder.like(root.get("reason").as(String.class), "%" + carParam.getReason() + "%"));
            }

            if (StrUtils.isNotNull(carParam.getCreateTimeStart()) && StrUtils.isNotNull(carParam.getCreateTimeEnd())) {
                //来访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get("createTime").as(Date.class),
                        DateUtils.toDate(carParam.getCreateTimeStart()),
                        DateUtils.toDate(carParam.getCreateTimeEnd())
                    )
                );
            }

            if (StrUtils.isNotBlank(carParam.getLabelCard())) {
                //标签卡号查询
                list.add(
                    criteriaBuilder.like(root.get("labelCard").as(String.class), "%" + carParam.getLabelCard() + "%")
                );
            }

            if (StrUtils.isNotBlank(carParam.getLabelCardStatus())) {
                //标签卡状态查询
                list.add(
                    criteriaBuilder.equal(root.get("labelCardStatus").as(String.class), carParam.getLabelCardStatus())
                );
            }

            if (
                StrUtils.isNotNull(carParam.getCardRecTimeStart()) && StrUtils.isNotNull(carParam.getCardRecTimeEnd())
            ) {
                //来访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get("cardRecTime").as(Date.class),
                        DateUtils.toDate(carParam.getCardRecTimeStart()),
                        DateUtils.toDate(carParam.getCardRecTimeEnd())
                    )
                );
            }

            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }
}
