/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.video.impl;

import com.yiring.app.domain.video.Video;
import com.yiring.app.domain.video.VideoRepository;
import com.yiring.app.param.video.VideoParam;
import com.yiring.app.param.video.VideoQueryParam;
import com.yiring.app.service.video.VideoService;
import com.yiring.app.util.GeoUtils;
import com.yiring.app.vo.video.VideoVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.util.StrUtils;
import com.yiring.common.vo.PageVo;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import org.locationtech.jts.geom.Point;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

/**
 * @author tzl
 * 2022/4/8 15:28
 */
@Service
public class VideoServiceImpl implements VideoService {

    @Resource
    VideoRepository videoRepository;

    @Override
    public Result<String> saveVideo(VideoParam videoParam) {
        if (has(videoParam.getUuid())) {
            return Result.no(Status.BAD_REQUEST, "您输入的编号已存在");
        }

        Video video = Video
            .builder()
            .createTime(LocalDateTime.now())
            //            .enable(videoParam.getEnable())
            //            .m3u8(videoParam.getM3u8())
            //            .point(videoParam.getPoint())
            .m3u8(videoParam.getM3u8())
            .point(GeoUtils.createPoint(videoParam.getLon(), videoParam.getLat()))
            //            .online(videoParam.getOnline())
            //            .type(videoParam.getType())
            //默认正常
            .videoName(videoParam.getVideoName())
            .status("1")
            .remark(videoParam.getRemark())
            .uuid(videoParam.getUuid())
            .build();
        videoRepository.saveAndFlush(video);
        //还需要同步添加国标28181中对应的摄像头信息
        //标识uuid对应流id,m3u8对应拉流地址,如m3u8中有标注 则将类型切换为FFmpeg,摄像头名称videoName对应名称,流名称默认"stream",超时时间默认60

        return Result.ok();
    }

    @Override
    public Result<String> deleteVideo(IdParam idParam) {
        Optional<Video> optional = videoRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Video video = optional.get();
        videoRepository.delete(video);
        //还需要同步删除国标28181中对应的摄像头信息

        return Result.ok();
    }

    @Override
    public Result<String> updateVideo(VideoParam videoParam, IdParam idParam) {
        Optional<Video> optional = videoRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Video video = optional.get();
        if (!video.getUuid().equals(videoParam.getUuid())) {
            //当修改编号时
            if (has(videoParam.getUuid())) {
                return Result.no(Status.BAD_REQUEST, "您输入的编号已存在");
            }
        }
        BeanUtils.copyProperties(videoParam, video);
        videoRepository.saveAndFlush(video);
        //还需要同步修改国标28181中对应的摄像头信息

        return Result.ok();
    }

    @Override
    public Result<Point> selectPoint(IdParam idParam) {
        Optional<Video> optional = videoRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Video video = optional.get();
        return Result.ok(video.getPoint());
    }

    @Override
    public Result<PageVo<VideoVo>> pageVideo(VideoQueryParam videoQueryParam, PageParam param) {
        Page<Video> all = videoRepository.findAll(condition(videoQueryParam), PageParam.toPageable(param));
        List<VideoVo> data = all
            .get()
            .map(video -> {
                VideoVo vo = new VideoVo();
                BeanUtils.copyProperties(video, vo);
                return vo;
            })
            .collect(Collectors.toList());
        PageVo<VideoVo> vo = PageVo.build(data, all.getTotalElements());
        return Result.ok(vo);
    }

    /**
     * 条件查询
     * @param videoQueryParam 筛选条件
     * @return Specification
     */
    public Specification<Video> condition(VideoQueryParam videoQueryParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (StrUtils.isNotBlank(videoQueryParam.getVideoName())) {
                list.add(
                    criteriaBuilder.like(
                        root.get("videoName").as(String.class),
                        "%" + videoQueryParam.getVideoName() + "%"
                    )
                );
            }

            if (StrUtils.isNotBlank(videoQueryParam.getUuid())) {
                list.add(
                    criteriaBuilder.like(root.get("uuid").as(String.class), "%" + videoQueryParam.getUuid() + "%")
                );
            }

            if (StrUtils.isNotBlank(videoQueryParam.getStatus())) {
                list.add(criteriaBuilder.equal(root.get("uuid").as(String.class), videoQueryParam.getStatus()));
            }
            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }

    boolean has(String uuid) {
        Video video = Video.builder().uuid(uuid).build();
        return videoRepository.count(Example.of(video)) > 0;
    }
}
