/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.service.post.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.auth.domain.post.Post;
import com.yiring.auth.domain.post.PostRepository;
import com.yiring.auth.excel.post.PostExcel;
import com.yiring.auth.param.post.PostFindParam;
import com.yiring.auth.param.post.PostParam;
import com.yiring.auth.service.post.PostService;
import com.yiring.auth.vo.post.PostVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.Executors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 职位信息
 * @author LJ-2204
 * @date 2022/4/13
 */
@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class PostServiceImpl implements PostService {

    @Resource
    PostRepository postRepository;

    @Override
    public Result<String> addPost(PostParam postParam) {
        if (hasPostInfo(postParam.getName())) return Result.no(Status.BAD_REQUEST, "职位已存在");
        Post post = postParam.transform();
        postRepository.save(post);
        return Result.ok();
    }

    @Override
    public Result<String> modifyPost(PostParam postParam, IdParam idParam) {
        Optional<Post> postOptional = postRepository.findById(idParam.getId());
        if (postOptional.isEmpty()) return Result.no(Status.BAD_REQUEST, "被修改的数据不存在");

        Post post = postOptional.get();

        if (!StrUtil.equals(post.getName(), postParam.getName())) {
            if (hasPostInfo(postParam.getName())) return Result.no(Status.BAD_REQUEST, "职位已存在");
        }

        post = postParam.transform();
        post.setId(idParam.getId());
        postRepository.save(post);
        return Result.ok();
    }

    @Override
    public Result<String> deletePost(IdParam idParam) {
        Post post = Post.builder().id(idParam.getId()).build();
        postRepository.delete(post);
        return Result.ok();
    }

    @Override
    public Result<PageVo<PostVo>> findPostPage(PostFindParam param) {
        Specification<Post> specification = getPostPageSpecification(param);

        if (ObjectUtil.isEmpty(param.getPageSize()) && ObjectUtil.isEmpty(param.getPageNo())) {
            //分页
            Pageable pageable = PageRequest.of(param.getPageNo() - 1, param.getPageSize());
            Page<Post> page = postRepository.findAll(specification, pageable);
            List<PostVo> postVos = PostVo.transforms(page.getContent());
            PageVo<PostVo> pageVo = PageVo.build(postVos, postVos.size());
            return Result.ok(pageVo);
        } else {
            List<Post> posts = postRepository.findAll(specification);
            List<PostVo> postVos = PostVo.transforms(posts);
            PageVo<PostVo> pageVo = PageVo.build(postVos, postVos.size());
            return Result.ok(pageVo);
        }
    }

    @Override
    public Result<PostVo> findPostById(IdParam idParam) {
        Post post = postRepository.getById(idParam.getId());
        PostVo postVo = PostVo.transform(post);
        return Result.ok(postVo);
    }

    @Override
    public boolean hasPostInfo(String name) {
        Post post = Post.builder().name(name).build();
        return postRepository.count(Example.of(post)) > 0;
    }

    @Override
    public void exportPostInfo(PostFindParam param, HttpServletResponse response) {
        Specification<Post> specification = getPostPageSpecification(param);
        List<Post> postList = postRepository.findAll(specification);
        List<PostExcel> postExcels = PostExcel.transforms(postList);
        try (
            DefaultStreamExcelBuilder<PostExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(PostExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(postExcels);

            String fileName = URLEncoder.encode("职位信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出岗位信息失败: " + e.getMessage());
        }
    }

    private Specification<Post> getPostPageSpecification(PostFindParam param) {
        return (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(param.getName())) {
                predicates.add(cb.equal(root.get(Post.Fields.name), param.getName()));
            }

            if (ObjectUtil.isNotEmpty(param.getEnable())) {
                predicates.add(cb.equal(root.get(Post.Fields.enable), param.getEnable()));
            }

            return cq.where(predicates.toArray(new Predicate[0])).getRestriction();
        };
    }
}
